diversityPlot.family<-function(data, samples=1, iterations=100, max.sample.size, step.size,
                               col, show.legend=TRUE, legend.names, 
			       smooth.curves=TRUE, print.diagonal=FALSE, col.diagonal="grey", lwd.diagonal=2,
			       ylim, xlab, ylab, lwd.lines=2, cex.lab=1, cex.axis=1, lwd.axis=1, cex.legend=1.5, ...){

  if(missing(max.sample.size)){
    max.sample.size<-max(apply(as.matrix(data$Data[,samples]), 2, sum))*2
  }
  
  if(missing(step.size)){
    step.size<-max.sample.size/100
  }

  if(missing(legend.names)){
    legend.names=colnames(data$Data)[samples]
  }
  
  if(missing(xlab)){
    xlab<-"Sampled reads"
  }
  
  if(missing(ylab)){
    ylab<-"Number of unique gene families"
  }

  x.seq<-seq(1, max.sample.size, by=step.size)  
  n.samples<-length(samples)
  
  if(missing(col)){
    col=1:n.samples
  }
    
  curves<-calculateRarefaction(data, samples=samples, iterations=iterations, x.seq=x.seq)

  y.max<-max(unlist(curves))
  
  if(missing(ylim)){
    ylim=c(0, y.max)
  }

  plot(NULL, xlim=c(0,max.sample.size), ylim=ylim, axes=FALSE, xlab=xlab, ylab=ylab, cex.lab=cex.lab, ...)
  axis(1, cex.axis=cex.axis, lwd=lwd.axis)
  axis(2, cex.axis=cex.axis, lwd=lwd.axis)
  
  if(print.diagonal==TRUE){
    abline(0, 1, col=col.diagonal, lwd=lwd.diagonal, lty=2)
  }
  
  for(s in 1:n.samples){ 
    if(smooth.curves==TRUE){
      lines(smooth.spline(x=x.seq, y=curves[[s]]), lwd=lwd.lines, col=col[s], lty=1)
    } else {
      lines(x=x.seq, y=curves[[s]], lwd=lwd.lines, col=col[s], lty=1)
    }
  }
  
  if(show.legend==TRUE){
    legend(x="bottomright", legend.names, col=col, lty=1, bty="n", cex=cex.legend, lwd=lwd.lines)
  }

  return()  				   
}

calculateRarefaction<-function(data, samples, iterations, x.seq){    
  
  if(sum(x.seq<1)>0){
    stop("Only positive (>0) values allowed.")
  }

  curves<-list()

  n.x.seq<-length(x.seq)
  x.max<-max(x.seq)
  
  n.samples<-length(samples)

  for(s in 1:n.samples){
  
    rarefaction.curve<-rep(0, times=length(x.seq))

    items<-rep(data$Annotation[,1], data$Data[,s])

    for(i in 1:iterations){
  
      samp<-sample(items, x.max, replace=TRUE)
      samp.cum<-cumsum(duplicated(samp)==FALSE)

      rarefaction.curve<-rarefaction.curve+samp.cum[x.seq]
  
    }

    rarefaction.curve<-rarefaction.curve/iterations
    
    curves[[s]]<-rarefaction.curve
  }
  
  return(curves)

}

estimateDiversity<-function(data, samples=1, method=c("chao", "simpson", "shannon")){

  n.samples<-length(samples)
  
  diversity<-list()

  for(s in 1:n.samples){
    diversity[[s]]<-list()
    
    
    for(m in method){

      # Chao species richness estimator
      if(m=="chao"){

	Sobs<-sum(data$Data[,samples[s]]>0)
	a<-sum(data$Data[,samples[s]]==1)
	b<-sum(data$Data[,samples[s]]==2)
      
        diversity[[s]]$Chao<-Sobs+(a^2/(2*b))
      
      } else if (m=="simpson"){
      
        s.obs<-data$Data[,samples[s]]>0
	N<-sum(data$Data[,samples[s]])
	D.1<-sum(data$Data[s.obs,s]*(data$Data[s.obs,s]-1))
	D.2<-N*(N-1)
	diversity[[s]]$Simpson<-1-D.1/D.2
      
      } else  if (m=="shannon"){
      
        N<-sum(data$Data[,samples[s]])
        abund<-data$Data[,samples[s]]/N
	s.obs<-data$Data[,samples[s]]>0
	
	diversity[[s]]$Shannon<-(-1)*sum(abund[s.obs]*log(abund[s.obs]))
      
      }         
    
    }
  
  }
  
  return(diversity)

}

clusterSamples<-function(data, samples=c(1,2,3), log.data=FALSE, method="complete", ...){

  myCol<-function(n){
    s<-seq(0,1,length=n)^2
    return(rgb(0,s,0))
  }

  mydist<-function(x){
    return(as.dist(1-cor(x)))
  }

  if(length(samples)<2){
    stop("The minimum number of samples needed for cluster analysis is two.")
  }

  if(log.data==TRUE){
    data.prep<-log(data$Data[,samples])
    tmp<-apply(data.prep, 2, is.infinite)
    data.prep[tmp]<-0
  } else { 
    data.prep<-data$Data[,samples]
  }

  data.dist<-mydist(data.prep)

  res.clust<-hclust(data.dist, method=method)
  plot(as.dendrogram(res.clust), ...)


}
