# This file contains standard IO stuff

readGeneFamilies<-function(file, type.family="detect", custom.annotation=NULL, sep="\t"){

  # Read the specified text file
  data<-read.table(file, header=TRUE, comment="", quote="", sep=sep)
  
  # Get the column names and their number
  cnames<-colnames(data)
  ncols<-dim(data)[2]
  nrows<-dim(data)[1]
  
  # The name of the first column needs to be GeneFamily. Otherwise stop
  if(cnames[1]!="GeneFamily"){
    stop(sprintf("Error reading %s: Column one needs to be named \"GeneFamily\"\n", file))
  }
    
  # What type of data do we have?
  if(tolower(type.family)=="detect"){
  
    # If type.family is set to detect we try to guess
    FamilyType<-detectGeneFamily(data[,"GeneFamily"])
    
    # Did we succeed in guessing the data type?
    if(is.null(FamilyType)){
      cat(sprintf("Could not detect gene family type. Using default (COG).\n"))
      FamilyType<-"COG"
    } else {
      cat(sprintf("Gene family type %s detected.\n", FamilyType))
    }
  } else {
    # Family type is specified
    FamilyType<-type.family
  }

  # Validate the data type
  res.familycheck<-checkFamilyType(data[, "GeneFamily"], type=FamilyType)
  
  # Was the data ok?
  if(res.familycheck==FALSE){
    stop("Failed to validate all entries in data file. Please make sure that the specified family type is correct.\n")
  }

  # Create the data struct
  DataStruct<-list()
  DataStruct$Data<-data[,2:ncols]
  DataStruct$Type<-FamilyType
  
  # Do we have a custom filetype?
  if(tolower(DataStruct$Type)=="custom"){
    # Has a custom annotation been specified?
    if(!is.null(custom.annotation)){
      # Add the custom annotation
      DataStruct$Annotation<-data.frame(as.character(data[,1]), addCustomAnnotation(as.character(data[,1]), custom.annotation))
    } else { 
      # Custom filetype without any annotation
      DataStruct$Annotation<-data.frame(as.character(data[,1]), rep(NA, nrows))
    }
  } else {
    # Known data type. Get the prestored annotation
    DataStruct$Annotation<-data.frame(as.character(data[,1]), getAnnotation(as.character(data[,1]), DataStruct$Type))
  }
  colnames(DataStruct$Annotation)<-c("GeneFamily", "Annotation")
  
  return(DataStruct)
}

detectGeneFamily<-function(gene.families){

  Patterns<-getSupportedFamilies()
  nPatterns<-length(Patterns)
  
  for(i in 1:nPatterns){

    res.match<-grep(Patterns[[i]], gene.families)

    if(length(res.match)>=length(gene.families)/2){
      return(names(Patterns)[i])
    }
  }
  
  return(NULL)  
}

checkFamilyType<-function(gene.families, type){

  if(tolower(type=="custom")){
    return(TRUE)
  }

  Patterns<-getSupportedFamilies()
  
  if(is.null(Patterns[[type]])){
    stop(sprintf("Unknown gene family type: %s.\n", type))
  }
  
  res.match<-grep(Patterns[[type]], gene.families)
  
  diff<-setdiff(1:length(gene.families), res.match)
  if(length(diff)>0){
    stop(sprintf("Error! The following rows could not be found in the gene family: %s.", paste(diff, collapse=",")))
  }
  
  return(TRUE)
				       
}

getAnnotation<-function(gene.families, type){

  if(missing(type)){
    stop("Gene family type is not specified.")
  }

  # Check for available annotation data
  if(!is.null(ShotgunFunctionalizeRAnnotationData$DESCRIPTIONS[[type]])){

    # Temporary data struct
    data.tmp<-data.frame(gene.families)
    colnames(data.tmp)<-as.character(type)
    
    # Merge with annotation data    
    Annotation<-merge(x=data.tmp, y=ShotgunFunctionalizeRAnnotationData$DESCRIPTIONS[[type]], by.x=type, by.y=type, all.x=TRUE, all.y=FALSE, sort=FALSE)
    
    ReOrder<-match(as.character(data.tmp[,1]), as.character(Annotation[,1]))
    Annotation<-Annotation[ReOrder,]

    return(as.character(Annotation[,2]))
    
  } else {
  
    # Return NA if no annotation is available
    return(rep(NA, length(gene.families)))
    
  }
}

addCustomAnnotation<-function(gene.families, annotation){

  if(ncol(annotation)!=2){
    stop("The custom annotation must have exactly two columns: custom gene families identifier and a description.")
  }

  data.tmp<-data.frame(gene.families)
  colnames(data.tmp)<-"GeneFamily"
  
  colnames(annotation)<-c("GeneFamily", "Description")
 
  
  Annotation<-merge(x=data.tmp, y=annotation, by.x="GeneFamily", by.y="GeneFamily", all.x=TRUE, all.y=FALSE, sort=FALSE)
  ReOrder<-match(as.character(data.tmp[,1]), as.character(Annotation[,1]))
  Annotation<-Annotation[ReOrder,]

  return(Annotation[,2])
  
}

getSupportedFamilies<-function(){
  Patterns<-list(COG="COG[0-9]", EC="EC[0-9]", ARGENE="ARGENE[0-9]", 
                 TIGRFAM="TIGR[0-9]", PFAM="PFAM[0-9]", PF="PF[0-9]")
  return(Patterns)
}

writeRankings<-function(Result, File, col.names=TRUE, sep="\t"){
  write.table(Result$Ranking, file=file, col.names=col.names, row.names=FALSE, quote=FALSE, sep=sep)
}

removeZeros<-function(data){

  sum.tot<-apply(data$Data, 1, sum)
  
  data.new<-data$Data[sum.tot>0,]
  annotation.new<-data$Annotation[sum.tot>0,]
  
  data.struct<-data
  data.struct$Data<-data.new
  data.struct$Annotation<-annotation.new
  
  return(data.struct)

}


getSamples<-function(data, samples, exclude=FALSE, removeZeros=FALSE){

  if(checkDataStructure(data)==FALSE){
    stop("Invalid data structure (data).")
  }

  newdata<-data
  n.samples<-dim(data$Data)[2]
  
  if(exclude==FALSE){
    samples.select<-samples   
  } else {
    samples.selecte<-setdiff(1:n.samples, samples)
  }
   
  newdata$Data<-as.data.frame(data$Data[,samples.select])
  colnames(newdata$Data)<-colnames(data$Data)[samples.select]
  
  if(removeZeros==TRUE){
    newdata<-removeZeros(newdata)
  }
  
  return(newdata)

}

# Merges two samples
mergeSamples<-function(data1, data2){

  if(checkDataStructure(data1)==FALSE){
    stop("Invalid data structure (data1).")
  }
  
  if(checkDataStructure(data2)==FALSE){
    stop("Invalid data structure (data2).")
  }

  samples.1<-colnames(data1$Data)
  samples.2<-colnames(data2$Data)
  
  n.1<-length(samples.1)
  n.2<-length(samples.2)
  
  samples.common<-intersect(samples.1, samples.2)
  
  if(length(samples.common)>0){
    stop(sprintf("Common samples: %s\n", samples.common))
  }
  
  # Generate a list of the common identifiers
  Annotation.new<-unique(rbind(data1$Annotation, data2$Annotation))
  
  # Common gene-families
  families.common<-as.character(Annotation.new[,1])
  
  nr<-dim(Annotation.new)[1] 
 
  Data.new.1<-matrix(0, nr=nr, nc=n.1)
  Data.new.2<-matrix(0, nr=nr, nc=n.2)
  
  m.1<-match(as.character(data1$Annotation[,1]), families.common)
  m.2<-match(as.character(data2$Annotation[,1]), families.common)
    
  Data.new.1[m.1,]<-as.matrix(data1$Data)
  Data.new.2[m.2,]<-as.matrix(data2$Data)
  
  Data.new<-cbind(Data.new.1, Data.new.2)
  colnames(Data.new)<-c(colnames(data1$Data), colnames(data2$Data))
  
  return(data.merged=list(Data=Data.new, Annotation=Annotation.new))

}

countCategories<-function(data, category){

  cat<-getCategory(category, data$Type)
  
  n.cat<-length(cat)
  n.samples<-dim(data$Data)[2]
  
  counts<-matrix(nr=n.cat, nc=n.samples)
  
  for(i in 1:n.cat){
    tmp<-data$Annotation[,1]%in%cat[[i]]
    counts[i,]<-apply(data$Data[tmp,], 2, sum)
  }
  colnames(counts)<-colnames(data$Data)

  result<-data.frame(Category=names(cat), counts)
  
  return(result)

}

checkDataStructure<-function(data){

  if(class(data)!="list"){
    return(FALSE)
  }

  if(is.null(data$Data) | is.null(data$Annotation)){
    return(FALSE)
  } else { 
    return(TRUE)
  }
}
